@@ -5,7 +5,7 @@
 
 package Graph::Easy::As_graphviz;
 
-$VERSION = '0.31';
+$VERSION = '0.74';
 
 #############################################################################
 #############################################################################
@@ -13,6 +13,9 @@ $VERSION = '0.31';
 package Graph::Easy;
 
 use strict;
+use warnings;
+
+use Graph::Easy::Util qw(ord_values);
 
 my $remap = {
   node => {
@@ -482,7 +485,7 @@ sub _att_as_graphviz
   my ($self, $out) = @_;
 
   my $att = '';
-  for my $atr (keys %$out)
+  for my $atr (sort keys %$out)
     {
     my $v = $out->{$atr};
     $v =~ s/\n/\\n/g;
@@ -512,6 +515,8 @@ sub _att_as_graphviz
   $att;
   }
 
+use Graph::Easy::Util qw(first_kv);
+
 sub _generate_group_edge
   {
   # Given an edge (from/to at least one group), generate the graphviz code
@@ -527,8 +532,8 @@ sub _generate_group_edge
   if ($from->isa('Graph::Easy::Group'))
     {
     # find an arbitray node inside the group
-    my ($n, $v) = each %{$from->{nodes}};
-    
+        my ($n, $v) = first_kv($from->{nodes});
+
     $a = 'ltail="cluster' . $from->{id}.'"';	# ltail=cluster0
     $from = $v;
     }
@@ -538,8 +543,8 @@ sub _generate_group_edge
   if ($to->isa('Graph::Easy::Group'))
     {
     # find an arbitray node inside the group
-    my ($n, $v) = each %{$to->{nodes}};
-    
+    my ($n, $v) = first_kv($to->{nodes});
+
     $b = 'lhead="cluster' . $to->{id}.'"';	# lhead=cluster0
     $to = $v;
     }
@@ -721,6 +726,91 @@ sub _generate_edge
   $txt . "$indent$first $self->{_edge_type} $other$edge_att\n";		# return edge text
   }
 
+sub _order_group 
+  {
+  my ($self,$group) = @_;
+  $group->{_order}++;
+  for my $sg (ord_values( $group->{groups}))
+	{
+		$self->_order_group($sg);
+	}
+  }
+
+
+sub _as_graphviz_group 
+  {
+  my ($self,$group) = @_;
+
+  my $txt = '';
+    # quote special chars in group name
+    my $name = $group->{name}; $name =~ s/([\[\]\(\)\{\}\#"])/\\$1/g;
+
+   return if $group->{_p};
+    # output group attributes first
+    my $indent = '  ' x ($group->{_order});
+    $txt .= $indent."subgraph \"cluster$group->{id}\" {\n${indent}label=\"$name\";\n";
+
+	for my $sg (ord_values ( $group->{groups} ))
+	{
+		#print '--'.$sg->{name}."\n";
+		$txt .= $self->_as_graphviz_group($sg,$indent);
+		$sg->{_p} = 1;
+	}
+    # Make a copy of the attributes, including our class attributes:
+    my $copy = {};
+    my $attribs = $group->get_attributes();
+
+    for my $key (sort keys %$attribs)
+      {
+      $copy->{$key} = $attribs->{$key};
+      }
+    # set some defaults
+    $copy->{'borderstyle'} = 'solid' unless defined $copy->{'borderstyle'};
+
+    my $out = $self->_remap_attributes( $group->class(), $copy, $remap, 'noquote');
+
+    # Set some defaults:
+    $out->{fillcolor} = '#a0d0ff' unless defined $out->{fillcolor};
+    $out->{labeljust} = 'l' unless defined $out->{labeljust};
+
+    my $att = '';
+    # we need to output style first ("filled" and "color" need come later)
+    for my $atr (reverse sort keys %$out)
+      {
+      my $v = $out->{$atr};
+      $v = '"' . $v . '"' if $v !~ /^[a-z0-9A-Z]+\z/;	# quote if nec.
+
+      # convert "x-dot-foo" to "foo". Special case "K":
+      my $name = $atr; $name =~ s/^x-dot-//; $name = 'K' if $name eq 'k';
+
+      $att .= $indent."$name=$v;\n";
+      }
+    $txt .= $att . "\n" if $att ne '';
+ 
+    # output nodes (w/ or w/o attributes) in that group
+    for my $n ($group->sorted_nodes())
+      {
+      # skip nodes that are relativ to others (these are done as part
+      # of the HTML-like label of their parent)
+      next if $n->{origin};
+
+      my $att = $n->attributes_as_graphviz();
+      $n->{_p} = undef;			# mark as processed
+      $txt .= $indent . $n->as_graphviz_txt() . $att . "\n";
+      }
+
+    # output node connections in this group
+    for my $e (ord_values $group->{edges})
+      {
+      next if exists $e->{_p};
+      $txt .= $self->_generate_edge($e, $indent);
+      }
+
+    $txt .= $indent."}\n";
+   
+   return $txt;
+  }
+
 sub _as_graphviz
   {
   my ($self) = @_;
@@ -801,67 +891,14 @@ sub _as_graphviz
   $self->_edges_into_groups() if $groups > 0;
 
   # output the groups (aka subclusters)
-  my $indent = '    ';
-  for my $group (sort { $a->{name} cmp $b->{name} } values %{$self->{groups}})
-    {
-    # quote special chars in group name
-    my $name = $group->{name}; $name =~ s/([\[\]\(\)\{\}\#"])/\\$1/g;
-
-    # output group attributes first
-    $txt .= "  subgraph \"cluster$group->{id}\" {\n${indent}label=\"$name\";\n";
-   
-    # Make a copy of the attributes, including our class attributes:
-    my $copy = {};
-    my $attribs = $group->get_attributes();
-
-    for my $a (keys %$attribs)
-      {
-      $copy->{$a} = $attribs->{$a};
-      }
-    # set some defaults
-    $copy->{'borderstyle'} = 'solid' unless defined $copy->{'borderstyle'};
-
-    my $out = $self->_remap_attributes( $group->class(), $copy, $remap, 'noquote');
-
-    # Set some defaults:
-    $out->{fillcolor} = '#a0d0ff' unless defined $out->{fillcolor};
-    $out->{labeljust} = 'l' unless defined $out->{labeljust};
-
-    my $att = '';
-    # we need to output style first ("filled" and "color" need come later)
-    for my $atr (reverse sort keys %$out)
-      {
-      my $v = $out->{$atr};
-      $v = '"' . $v . '"' if $v !~ /^[a-z0-9A-Z]+\z/;	# quote if nec.
-
-      # convert "x-dot-foo" to "foo". Special case "K":
-      my $name = $atr; $name =~ s/^x-dot-//; $name = 'K' if $name eq 'k';
-
-      $att .= "    $name=$v;\n";
-      }
-    $txt .= $att . "\n" if $att ne '';
- 
-    # output nodes (w/ or w/o attributes) in that group
-    for my $n ($group->sorted_nodes())
-      {
-      # skip nodes that are relativ to others (these are done as part
-      # of the HTML-like label of their parent)
-      next if $n->{origin};
-
-      my $att = $n->attributes_as_graphviz();
-      $n->{_p} = undef;			# mark as processed
-      $txt .= $indent . $n->as_graphviz_txt() . $att . "\n";
-      }
-
-    # output node connections in this group
-    for my $e (values %{$group->{edges}})
-      {
-      next if exists $e->{_p};
-      $txt .= $self->_generate_edge($e, $indent);
-      }
-
-    $txt .= "  }\n";
-    }
+  for my $group (ord_values $self->{groups})
+  {
+   $self->_order_group($group);
+  }
+  for my $group (sort { $a->{_order} cmp $b->{_order} } values %{$self->{groups}})
+  {
+    $txt .= $self->_as_graphviz_group($group) || '';
+  }
 
   my $root = $self->attribute('root');
   $root = '' unless defined $root;
@@ -912,7 +949,7 @@ sub _as_graphviz
 
   # insert now edges between groups (clusters/subgraphs)
 
-  foreach my $e (values %{$self->{edges}})
+  foreach my $e (ord_values $self->{edges})
     {
     $txt .= $self->_generate_group_edge($e, '  ') 
      if $e->{from}->isa('Graph::Easy::Group') ||
@@ -920,7 +957,7 @@ sub _as_graphviz
     }
 
   # clean up
-  for my $n ( values %{$self->{nodes}}, values %{$self->{edges}})
+  for my $n ( ord_values( $self->{nodes}), ord_values( $self->{edges} ))
     {
     delete $n->{_p};
     }
