@@ -4,7 +4,7 @@
 
 package Graph::Easy::As_txt;
 
-$VERSION = '0.15';
+$VERSION = '0.74';
 
 #############################################################################
 #############################################################################
@@ -12,6 +12,7 @@ $VERSION = '0.15';
 package Graph::Easy;
 
 use strict;
+use warnings;
 
 sub _as_txt
   {
@@ -212,13 +213,13 @@ sub attributes_as_txt
     my $names = {};
     for my $child ($self, @$parts)
       {
-      for my $k (keys %{$child->{att}})
+      for my $k (sort keys %{$child->{att}})
         {
         $names->{$k} = undef;
         }
       }
 
-    for my $k (keys %$names)
+    for my $k (sort keys %$names)
       {
       next if $k eq 'basename';
       my $val = $self->{att}->{$k};
