@@ -5,9 +5,10 @@
 
 package Graph::Easy::Base;
 
-$VERSION = '0.12';
+$VERSION = '0.74';
 
 use strict;
+use warnings;
 
 #############################################################################
 
@@ -342,8 +343,8 @@ Create a new object, and call C<_init()> on it.
 
 Returns the last error message, or '' for no error.
 
-When setting a new error message, C<$self->_croak($error)> will be called
-unless C<$object->no_fatal_errors()> is true.
+When setting a new error message, C<< $self->_croak($error) >> will be called
+unless C<< $object->no_fatal_errors() >> is true.
 
 =head2 error_as_html()
 
