@@ -7,9 +7,10 @@ package Graph::Easy::Edge;
 
 use Graph::Easy::Node;
 @ISA = qw/Graph::Easy::Node/;		# an edge is just a special node
-$VERSION = '0.31';
+$VERSION = '0.74';
 
 use strict;
+use warnings;
 
 use constant isa_cell => 1;
 
@@ -25,7 +26,7 @@ sub _init
   # leave this unitialized until we need it
   # $self->{cells} = [ ];
 
-  foreach my $k (keys %$args)
+  foreach my $k (sort keys %$args)
     {
     if ($k !~ /^(label|name|style)\z/)
       {
