@@ -7,10 +7,13 @@
 package Graph::Easy::Layout::Chain;
 
 use Graph::Easy::Base;
-$VERSION = '0.09';
+$VERSION = '0.74';
 @ISA = qw/Graph::Easy::Base/;
 
 use strict;
+use warnings;
+
+use Graph::Easy::Util qw(ord_values);
 
 use constant {
   _ACTION_NODE  => 0, # place node somewhere
@@ -26,7 +29,7 @@ sub _init
   # Generic init routine, to be overriden in subclasses.
   my ($self,$args) = @_;
   
-  foreach my $k (keys %$args)
+  foreach my $k (sort keys %$args)
     {
     if ($k !~ /^(start|graph)\z/)
       {
@@ -175,7 +178,7 @@ sub layout
     {
     # first do edges going from P to N
     #for my $e (sort { $a->{to}->{name} cmp $b->{to}->{name} } values %{$pre->{edges}})
-    for my $e (values %{$pre->{edges}})
+    for my $e (ord_values ( $pre->{edges}))
       {
       # skip selfloops and backward links, these will be done later
       next if $e->{to} != $n;
@@ -212,7 +215,7 @@ sub layout
 
     # gather all edges starting at $n, but do the ones with a flow first
 #    for my $e (sort { $a->{to}->{name} cmp $b->{to}->{name} } values %{$n->{edges}})
-    for my $e (values %{$n->{edges}})
+    for my $e (ord_values ( $n->{edges})) 
       {
       # skip selfloops, these will be done later
       next if $e->{to} == $n;
@@ -275,7 +278,7 @@ sub layout
   while (defined $n)
     {
 #    for my $e (sort { $a->{to}->{name} cmp $b->{to}->{name} } values %{$n->{edges}})
-    for my $e (values %{$n->{edges}})
+    for my $e (ord_values $n->{edges})
       {
       next unless exists $e->{_todo};
 
@@ -485,7 +488,7 @@ Return the length of the chain, in nodes.
 
 	my $len = $chain->length( $node );
 
-Given an optional C<$node> as argument, returns the lenght
+Given an optional C<$node> as argument, returns the length
 from that node onwards. For the chain with the three nodes
 A, B and C would return 3, 2, and 1 for A, B and C, respectively.
 
