@@ -5,7 +5,7 @@
 
 package Graph::Easy::Layout::Path;
 
-$VERSION = '0.16';
+$VERSION = '0.74';
 
 #############################################################################
 #############################################################################
@@ -13,6 +13,7 @@ $VERSION = '0.16';
 package Graph::Easy::Node;
 
 use strict;
+use warnings;
 
 use Graph::Easy::Edge::Cell qw/
  EDGE_END_E EDGE_END_N EDGE_END_S EDGE_END_W
@@ -388,6 +389,8 @@ sub _placed_shared
   $placed;
   }
 
+use Graph::Easy::Util qw(first_kv);
+
 sub _find_node_place
   {
   # Try to place a node (or node cluster). Return score (usually 0).
@@ -434,7 +437,7 @@ sub _find_node_place
   # if the node has outgoing edges (which might be shared)
   if (!ref($edge))
     {
-    (undef,$edge) = each %{$node->{edges}} if keys %{$node->{edges}} > 0;
+    (undef,$edge) = first_kv($node->{edges}) if keys %{$node->{edges}} > 0;
     }
 
   my $dir = undef; $dir = $edge->flow() if ref($edge);
