@@ -5,11 +5,12 @@
 
 package Graph::Easy::Parser::VCG;
 
-$VERSION = '0.06';
+$VERSION = '0.74';
 use Graph::Easy::Parser::Graphviz;
 @ISA = qw/Graph::Easy::Parser::Graphviz/;
 
 use strict;
+use warnings;
 use utf8;
 use constant NO_MULTIPLES => 1;
 use Encode qw/decode/;
@@ -338,12 +339,12 @@ sub _new_scope
     my $old_scope = $self->{scope_stack}->[-1];
 
     # make a copy of the old scope's attribtues
-    for my $t (keys %$old_scope)
+    for my $t (sort keys %$old_scope)
       {
       next if $t =~ /^_/;
       my $s = $old_scope->{$t};
       $scope->{$t} = {} unless ref $scope->{$t}; my $sc = $scope->{$t};
-      for my $k (keys %$s)
+      for my $k (sort keys %$s)
         {
         # skip things like "_is_group"
         $sc->{$k} = $s->{$k} unless $k =~ /^_/;
@@ -500,7 +501,7 @@ sub _build_match_stack
       $scope->{$type} = {} unless ref $scope->{$type};
       my $s = $scope->{$type};
 
-      for my $k (keys %$att)
+      for my $k (sort keys %$att)
         {
         $s->{$k} = $att->{$k};
         }
@@ -754,7 +755,7 @@ my $vcg_remap = {
   # add all graph attributes to group, too
   my $group = $vcg_remap->{group};
   my $graph = $vcg_remap->{graph};
-  for my $k (keys %$graph)
+  for my $k (sort keys %$graph)
     {
     $group->{$k} = $graph->{$k};
     }
@@ -961,7 +962,7 @@ sub _remap_attributes
 #    use Data::Dumper; print Dumper($att);
 
   # handle the "colorentry 00" entries:
-  for my $key (keys %$att)
+  for my $key (sort keys %$att)
     {
     if ($key =~ /^colorentry\s+([0-9]{1,2})/)
       {
